@@ -63,8 +63,12 @@ static const R_ExternalMethodDef ExtEntries[] = {
 };
 
 
+#include <Rversion.h>
 void R_init_foreign(DllInfo *dll)
 {
     R_registerRoutines(dll, CEntries, CallEntries, NULL, ExtEntries);
     R_useDynamicSymbols(dll, FALSE);
+#if defined(R_VERSION) && R_VERSION >= R_Version(3, 0, 0)
+    R_forceSymbols(dll, TRUE);
+#endif
 }
